@@ -7,7 +7,7 @@ use File::Copy;
 use File::Path;
 use File::Spec;
 
-my ($sourcedir, $libdir) = @ARGV;
+my ($sourcedir, $libdir, $verbose) = @ARGV;
 
 foreach ($sourcedir, $libdir) { s,/$,,; }
 
@@ -71,11 +71,11 @@ sub merge
     open TO, ">$t_out" or die "Can't write to '$t_out': $!";
     print TO $to or die "Can't write to '$t_out': $!";
     close TO or die "Can't write to '$t_out': $!";
-    chmod 0444, $t_out or die "Can't chmod '$t_out' to 444: $!";
+    chmod 0644, $t_out or die "Can't chmod '$t_out' to 444: $!";
 
     for ( $file, $t_out ) { s,^.*(?=Alzabo),,; s/\.pm$//; s,[\\/],::,g; }
 
-    print STDERR "merged $file docs into $t_out\n";
+    print "merged $file docs into $t_out\n" if $verbose;
 }
 
 sub find_chunk
